local addonName, addon = ...

addon.config = {
	scale = 1,
	position = {"CENTER", UIParent, "CENTER", 300, 0},
	updateInterval = 1,
	
	consumables = {
		enabled = true,
		foodThresholdTime = 5,
		flaskThresholdTime = 5
	},
	
	druid = {
		enabled = true
	},
	
	general = {
		enabled = true
	},
	
	mage = {
		enabled = true,
		armors = {"Molten Armor", "Mage Armor"}
	},
	
	paladin = {
		enabled = true,
		auras = {"Devotion Aura", "Retribution Aura", "Crusader Aura", "Shadow Resistance Aura", "Frost Resistance Aura", "Fire Resistance Aura"},
		seals = {"Seal of Command", "Seal of Righteousness", "Seal of Justice", "Seal of Light", "Seal of Wisdom", "Seal of Corruption", "Seal of Vengeance"},
		blessings = {"Blessing of Might", "Greater Blessing of Might", "Blessing of Wisdom", "Greater Blessing of Wisdom", "Blessing of Kings", "Greater Blessing of Kings", "Blessing of Sanctuary"},
		righteousFury = true
	},

	priest = {
		enabled = true
	},
	
	rogue = {
		enabled = true,
		mainHandPoisons = {"Wound Poison", "Instant Poison"},
		offHandPoisons = {"Crippling Poison", "Mind-numbing Poison"},
		thresholdTime = 2
	},
	
	shaman = {
		enabled = true,
		shields = {"Lightning Shield", "Water Shield"},
		mainHandEnchants = {"Flametongue Weapon", "Windfury Weapon"},
		offHandEnchants = {"Flametongue Weapon", "Windfury Weapon"},
		thresholdTime = 2
	},
	
	warlock = {
		enabled = true,
		armors = {"Demon Skin", "Fel Armor"},
		thresholdTime = 10
	},
}

addon.playerClass = select(2, UnitClass("player"))

local config = addon.config
local frame = CreateFrame("Frame", nil, UIParent)
local reminders = {}

local onEvent = function(self, event)
	frame:SetScale(config.scale)
	frame:SetPoint(unpack(config.position))
end

local lastUpdate = 0
local updateInterval = config.updateInterval
local onUpdate = function(self, elapsed)
	lastUpdate = lastUpdate + elapsed
	
	if lastUpdate > updateInterval then
		lastUpdate = 0
		
		addon:UpdateReminders()
	end
end

local onEnter = function(self)
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetWidth(250)
	GameTooltip:AddLine(self.name)

	if self.tooltip then
		GameTooltip:AddLine(" ")
		GameTooltip:AddLine(self.tooltip)
	else
		local left = self:GetAttribute("item1")
		if left then
			GameTooltip:AddLine(" ")
			GameTooltip:AddLine("Left-click to use " .. left)
		else
			left = self:GetAttribute("spell1")
			if left then
				GameTooltip:AddLine(" ")
				GameTooltip:AddLine("Left-click to cast " .. left)
			end
		end

		local right = self:GetAttribute("item2")
		if right then
			if not left then
				GameTooltip:AddLine(" ")
			end

			GameTooltip:AddLine("Right-click to use " .. right)
		else
			right = self:GetAttribute("spell2")
			if right then
				if not left then
					GameTooltip:AddLine(" ")
				end

				GameTooltip:AddLine("Right-click to cast " .. right)
			end
		end
	end

	GameTooltip:Show()
end

local onLeave = function(self)
	GameTooltip:Hide()
end

local disableReminder = function(self, reminder, reactivateTime)
	reminder.active = false
	reminder.reactivateTime = reactivateTime and (GetTime() + reactivateTime) or 0
end

local menu
local menuFrame = CreateFrame("Frame", addonName .. "Menu", UIParent, "UIDropDownMenuTemplate")
local onClick = function(self)
	menu = {
		{text = self.name, isTitle = true},
		{text = "Suppress for 5 minutes", func = disableReminder, arg1 = self, arg2 = 300},
		{text = "Disable for this session", func = disableReminder, arg1 = self}
	}

	EasyMenu(menu, menuFrame, "cursor", nil, nil, "MENU")
end

-- Utility functions
function addon:IsInPartyWith(class)
	local numMembers = GetNumPartyMembers()

	if numMembers then
		for i = 1, numMembers do
			local unit = "party" .. i

			if not UnitIsPlayer("unit") and select(2, UnitClass(unit)) == class then
				return true
			end
		end
	end

	return false
end

function addon:IsBigWigsModuleActive(name)
	local bw = _G.BigWigs

	return bw and bw:IsModuleActive(name)
end

function addon:PlayerHasBuff(name)
	return UnitAura("player", name, nil, "HELPFUL")
end

function addon:PlayerHasAnyAura(names, filter)
	if not filter then
		filter = "HELPFUL"
	end

	for _, name in ipairs(names) do
		if UnitAura("player", name, nil, filter) then
			return true
		end
	end
	
	return false
end

function addon:PlayerInPVEInstance()
	isInstance, instanceType = IsInInstance()
	
	if isInstance then
		return instanceType == "party" or instanceType == "raid"
	end
	
	return false
end

function addon:HasTalent(tabIndex, talentIndex, rankRequired)
	return select(5, GetTalentInfo(tabIndex, talentIndex)) >= (rankRequired or 1)
end

function addon:HasGlyph(id)
	for i = 1, 6 do
		local _, _, glyphSpell = GetGlyphSocketInfo(i)
		
		if glyphSpell == id then
			return true
		end
	end
	
	return false
end

function addon:AddReminder(name, callback, icon, attributes, tooltip, color)
	local buttonName = "ReminderButton" .. #reminders
	local frame = CreateFrame("Button", buttonName, frame, "SecureActionButtonTemplate, ActionButtonTemplate")

	frame.name = name
	frame.callback = callback
	frame.attributes = attributes
	frame.tooltip = tooltip
	frame.active = true
	frame.reactivateTime = 0

	local texture = frame:CreateTexture(nil, "BACKGROUND")
	texture:SetAllPoints(frame)
	texture:SetTexture("Interface\\Icons\\" .. (icon or "Temp"))
	texture:SetTexCoord(.07, .93, .07, .93)
	
	if color then
		texture:SetVertexColor(unpack(color))
	end

	_G[frame:GetName() .. "Icon"]:SetTexture(texture)

	frame:RegisterForClicks("AnyUp")
	frame:SetScript("OnEnter", onEnter)
	frame:SetScript("OnLeave", onLeave)

	if attributes then
		for key, value in pairs(attributes) do
			frame:SetAttribute(key, value)
		end
	end

	frame:SetAttribute("alt-type*", "showmenu")
	frame.showmenu = onClick
	
	table.insert(reminders, frame)

	return frame
end

function addon:UpdateReminders()
	local result
	local previousReminder
	local inCombat = InCombatLockdown()

	for _, reminder in ipairs(reminders) do
		if not reminder.active and (reminder.reactivateTime > 0 and GetTime() >= reminder.reactivateTime) then
			reminder.active = true
			reminder.reactivateTime = 0
		end

		result = reminder.active and reminder.callback()

		if result then
			if not inCombat then
				reminder:ClearAllPoints()

				if previousReminder then
					reminder:SetPoint("LEFT", previousReminder, "RIGHT", 5, 0)
				else
					reminder:SetPoint("TOPLEFT", frame)
				end

				reminder:SetAlpha(1)
				reminder:Show()
			end

			previousReminder = reminder
		elseif inCombat then
			reminder:SetAlpha(0)
		else
			reminder:Hide()
		end
	end
end

frame:SetWidth(36)
frame:SetHeight(36)
frame:SetScript("OnEvent", onEvent)
frame:SetScript("OnUpdate", onUpdate)

frame:RegisterEvent("PLAYER_ENTERING_WORLD")
